@@ -1,91 +1,39 @@
-/* Written by and copyright Maurizio Sartori <masar@MasarLabs.com>. */
-
-/* seek: extract rows at selected offsets from a large, indexed table.
-
-   Copyright (C) 1998-2001 Maurizio Sartori <masar@MasarLabs.com>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-
-
-   Usage:  seek [options] table < offset_list
-
-   Options:
-       -R|--raw-input
-            The input offset list is not in table format.
-
-       -x|--hex
-            Input offset are expressed in hexadecimal values.
-
-       -N|--no-header
-            Do not print the table header on STDOUT.
-
-       -i|--info
-            Instead of the actual table content, a new table is
-            printed, containing information on the requested offset(s).
-            The information printed is the start, end and size (in bytes)
-            of the relevant rows, and a flag that tells whether the
-            latter are empty, i.e. contain just blanks and/or tabs.
-
-       -b|--blank
-            Same as '--info', but only blank lines, if any, are taken into
-            account.
-
-
-   This operator takes a list of byte-offsets on STDIN and prints on STDOUT
-   the corresponding table records from the table specified as a command
-   line argument. The list of offsets can be built with the 'index'
-   operator, and 'seek' expects it to be a one-column table, like this:
-
-           lpos
-           ----
-           228
-           1117
-           518
-           1225
-
-   The column name, 'lpos' in the example, can take any name, provided
-   that the list has a valid NoSQL table format. If option '-R' is
-   given, then the offsets are expected to be a blank- , tab- or
-   newline-separated raw list of numbers on STDIN, i.e. they are not
-   expected to to have a valid NoSQL table header.
-
-   If option '-x' is given, then the offsets are expected to be base 16
-   numbers instead of the usual base 10 numbers.
-
-   Examples :
-
-     seek table < offset_table
-
-     or
-
-     echo "228 1117 518 1225" | seek -R table
-     echo "a28 1b1F 5c8 1425" | seek -R -x table
-
-
- ****************************************************************************/
-
 /*
-static char Psz_RCS [] =
-    "$Id: seek.c,v 1.1.1.1 2003/03/17 09:49:20 carlo Exp $";
-*/
+  seek.c: extract rows at selected offsets from an indexed
+  NoSQL table.
 
-/****************************************************************************/
+  Original code Copyright (c) 1998 Maurizio Sartori
+
+  Changes for NoSQL Copyright (c) 2004 Carlo Strozzi
+
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
+
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+  $Id: seek.c,v 1.2 2004/08/08 21:09:06 carlo Exp $
+
+*/
 
 #include <stdio.h>
 #include <ctype.h>
+#include <unistd.h>
 
+#ifndef HELPDIR
+#define HELPDIR "/usr/local/nosql/help"
+#endif
+#ifndef HELPFILE
+#define HELPFILE (HELPDIR "/seek.txt")
+#endif
 
 /****************************************************************************/
 /*                          Function prototypes                             */
@@ -156,10 +104,12 @@ void LineInfo( FILE * file, const int fBlank, const char * pszStr )
    {
      if ( cBlank == 'y' )
      {
-       printf( "%ld\t%ld\t%ld\t%c\n", iStart, iEnd, iSize, cBlank );
+       printf( "\001%ld\t\001%ld\t\001%ld\t\001%c\n", \
+				iStart, iEnd, iSize, cBlank );
      }
    }
-   else printf( "%ld\t%ld\t%ld\t%c\n", iStart, iEnd, iSize, cBlank );
+   else printf( "\001%ld\t\001%ld\t\001%ld\t\001%c\n", \
+				iStart, iEnd, iSize, cBlank );
 }
 
 /****************************************************************************/
@@ -191,6 +141,7 @@ int main( int argc, char **argv )
    int    fHdr = 1;
    int    fInfo = 0;
    int    fBlank = 0;
+   int	  some_input = 0;
    FILE * fileDb = NULL;
    int    iInx;
    int    chChar;
@@ -222,7 +173,7 @@ int main( int argc, char **argv )
       {
          fRaw = 1;
       }
-      else if ( !strcmp ( argv[iInx], "-x" )
+      else if ( !strcmp ( argv[iInx], "-X" )
             || !strcmp ( argv[iInx], "--hex" ) )
       {
          iValue['a'] = iValue['A'] = 11;
@@ -238,7 +189,7 @@ int main( int argc, char **argv )
       {
          fHdr = 0;
       }
-      else if ( !strcmp( argv[iInx], "-i" )
+      else if ( !strcmp( argv[iInx], "-r" )
         || !strcmp( argv[iInx], "--info" ) )
       {
          fInfo = 1;
@@ -249,6 +200,12 @@ int main( int argc, char **argv )
          fBlank = 1;
          fInfo = 1;
       }
+      else if (!strcmp(argv[iInx], "-h") ||
+	       !strcmp(argv[iInx], "--help")) {
+	      execlp("grep","grep","-v","^#",HELPFILE,(char *) 0);
+	      perror("grep");
+	      exit(0);
+      }
       else if ( fileDb )
       {
          fprintf( stderr, "seek: invalid number of parameters\n" );
@@ -272,21 +229,19 @@ int main( int argc, char **argv )
    }
 
    /*************************************************************************/
-   /* Put headers (first and second line of the database)                   */
+   /* Print table header                                                    */
    /*************************************************************************/
    if ( fHdr && !fInfo )
    {
-     LineOutput( fileDb, "Invalid Database Header 1\n" );
-     LineOutput( fileDb, "Invalid Database Header 2\n" );
+     LineOutput( fileDb, "Invalid Database Header\n" );
    }
 
    /*************************************************************************/
-   /* Skip headers if not Raw Mode                                          */
+   /* Skip header if not Raw Mode                                          */
    /*************************************************************************/
    if ( !fRaw )
    {
-      LineSkip( stdin, "seek: invalid input header 1\n" );
-      LineSkip( stdin, "seek: invalid input header 2\n" );
+      LineSkip( stdin, "seek: invalid input table header\n" );
    }
 
    /* Print the output table header if '--info' was requested. */
@@ -316,6 +271,8 @@ int main( int argc, char **argv )
                exit(1);
             }
 
+	    some_input = 1;
+
             if ( !fInfo )
               LineOutput( fileDb, "seek: invalid file line\n" );
             else
@@ -333,7 +290,8 @@ int main( int argc, char **argv )
       chChar = fgetc( stdin );
    }
 
-   return 0;
+   if (!some_input) exit(1);
+   else exit(0);
 }
 
 /* End of program */
